==> hs_export_data-1.0.6.tgz: package/export_plot.js <==
const xlsx = require("node-xlsx");
const path = require('path')
const fs = require('fs')
const base_path = __dirname;
var parseString = require('xml2js').parseString;
var pako = require('pako');

const myReplace = function(data, f, e) { //吧f替换成e
    var reg = new RegExp(f, "g"); //创建正则RegExp对象   
    return data.replace(reg, e);
}

const jsonfile = require('jsonfile')
let t_export_dis = `${base_path}/./../`;
let t_export_name = "config.json";
let t_target_dis = `${base_path}/excels`;
let ts_target_dir = `${base_path}/ts`;
let ts_import = "";

let server_data_config = [];

const server_config = {};

function exportdata(texport_dis, texport_name, ttarget_dis, ts_dir, server_data_config) {
        server_data_config = server_data_config;
        console.log("服务器端的配置数据", server_data_config);
        let export_dis = texport_dis;
        let export_name = texport_name;
        let target_dis = ttarget_dis;
        fs.readdir(target_dis, (error, files) => {
            console.log(target_dis);
            const total_data = {};
            var interfaces = "";
            var all_config = "const ConfigsName = {\n";

            var export_interface = " \n\n export { ";
            files.forEach((filename, index) => {
                let pathname = path.join(target_dis, filename);
                if (filename.indexOf(".DS_Store") == 0) {
                    return;
                }
                fs.stat(pathname, (err, stats) => { // 读取文件信息
                    if (err) {
                        return
                    }
    
                    if (stats.isFile()) {
    
                        console.log("pathname  ", pathname);
                        const list = xlsx.parse(fs.readFileSync(pathname));
                        const data = praseExcel(list, filename);

                        const tname = filename.split(".")[0];
                        ts_import = ts_import + `import {${tname}} from "./ConfigInterface" \n`;
                        
                        const ts = prase_ts(list, tname);
                        interfaces = interfaces + ts;
                        if (data) {
                            total_data[`${tname}`] = data;

                            let is_server_config = false;
                            for(const file_name of server_data_config){
                                if(file_name == tname){
                                    is_server_config = true;
                                    break;
                                }
                            }
    
                            if(is_server_config){
                                server_config[`${tname}`] = data;
                            }
                        }

                        
    
                        export_interface = export_interface + tname + ","
                        all_config = all_config + `     ${tname}:"${tname}", \n`;

                        if (index == files.length - 1) {
                            setTimeout(() => {
                                 let data_str = JSON.stringify(total_data);
                                 writeFile(path.join(export_dis, export_name), JSON.parse(data_str), null, "\t");   
                                 writeFile(path.join(export_dis, "ServerConfig.json"), JSON.parse(JSON.stringify(server_config)), null, "\t");   

                                 const tpath = path.join(ts_dir, "ConfigInterface.ts");
                                 console.log("tpath ",tpath)
                                 fs.writeFile(tpath, `${interfaces} \n ${export_interface} };`, 'utf-8', (err) => {
                                     if (!err)      {         
                                         console.log(`导出文件${tpath}成功`);     
                                     }   
                                 }); 

                                 
                                 const t1path = path.join(ts_dir, "ConfigsName.ts");
                                 fs.writeFile(t1path, `${ts_import} \n\n ${all_config} }; \n export default ConfigsName;`, 'utf-8', (err) => {
                                     if (!err)      {         
                                         console.log(`导出文件${tpath}成功`);     
                                     }   
                                 }); 



                            }, 100);
                        }
                    }
    
    
                });
            });
        });


}

function parse_ts(list) {}

function praseExcel(list) {   
    var sheetArray  = [];      
    for (var i = 0; i < 1; i++) {        
        var excleData = list[i].data;        
        var typeArray =  excleData[2];        
        var keyArray =  excleData[0];      
        for (var j = 3; j < excleData.length; j++) {            
            var curData = excleData[j];            
            if (curData.length == 0) continue;            
            var item = changeObj(curData, typeArray, keyArray);     
            // console.log(item);        
            sheetArray.push(item);       
        }       
        // if (sheetArray) {
        //     // console.log("sheetArray", sheetArray); 
        //     writeFile(path.join(`${base_path}/./../res`, list[i].name + ".json"), formatJson(JSON.stringify(sheetArray)), null, "\t");   
        // }  
    }  
    return sheetArray;
}

function changeObj(curData, typeArray, keyArray) {    
    const plot_single = {};
    const plot_arraay = [
        []
    ];
    for (var i = 0; i < curData.length; i++)    { 
        const key = keyArray[i];
        if (key && key.substr(0, 1) !== "#") {
            plot_single[key] = changeValue(curData[i], typeArray[i]); 
        }
    }   


    return plot_single;
}

function changeValue(value, type) {   
    if (value == null || value == "null") return ""   
    if (type == "bool" || type == "b") return value;   
    if (type == "int" || type == "i") return Math.floor(value);   
    if (type == "bool") return value;
    if (type == "Number" || type == "number" || type == "n" || type == "float") return new Number(value);   
    if (type == "String" || type == "s" || type == "S" || type == "string") return value;
    if (type == "json" || type == "object" || type == "Array<string>" || type == "Array<number>" || type == "Array<any>" || type == "array" || type == "A" || type == "a") return JSON.parse(value); 
}

function prase_ts(list, file_name) {   
      
    let ts = "";

    for (var i = 0; i < 1; i++) {        
        var excleData = list[i].data;        
        var typeArray =  excleData[2];        
        var keyArray =  excleData[0];      
        ts = ts + single_ts_config_interface(file_name, typeArray, keyArray);
    }  
    
   
    return ts;
}

function single_ts_config_interface(name, typeArray, keyArray){
     var ts_interface = `interface ${name} { \n`;
     let key_index = 0;

     for(const type of typeArray){
        const key = keyArray[key_index];

        ts_interface = ts_interface + `    ${key} : ${type}; \n`;

        key_index = key_index + 1;
     }

     ts_interface =  `${ts_interface}} \n\n`;

     return ts_interface;
}

function writeFile(fileName, data) {  
    // fs.writeFile(fileName, data, 'utf-8', (err) => {
    //     if (!err)      {         
    //         console.log(`导出文件${fileName}成功`);     
    //     }   
    // }); 

    jsonfile.writeFile(fileName, data, function(err) {
        if (err) console.error(err)
        console.log(`导出文件${fileName}成功`);
    })

    compress_data(data, t_export_dis , "config.manifest");
}

var formatJson = function(txt, compress) {
    var indentChar = '    ';
    if (/^\s*$/.test(txt)) {
        console.log('数据为空,无法格式化! ');
        return;
    }
    try {
        var data = eval('(' + txt + ')');
    } catch (e) {
        console.log('数据源语法错误,格式化失败! 错误信息: ' + e.description, 'err');
        return;
    }
    var draw = [],
        last = false,
        This = this,
        line = compress ? '' : '\n',
        nodeCount = 0,
        maxDepth = 0;

    var notify = function(name, value, isLast, indent, formObj) {
        nodeCount++; /*节点计数*/
        for (var i = 0, tab = ''; i < indent; i++)
            tab += indentChar; /* 缩进HTML */
        tab = compress ? '' : tab; /*压缩模式忽略缩进*/
        maxDepth = ++indent; /*缩进递增并记录*/
        if (value && value.constructor == Array) {
            /*处理数组*/
            draw.push(
                tab + (formObj ? '"' + name + '":' : '') + '[' + line
            ); /*缩进'[' 然后换行*/
            for (var i = 0; i < value.length; i++)
                notify(i, value[i], i == value.length - 1, indent, false);
            draw.push(
                tab + ']' + (isLast ? line : ',' + line)
            ); /*缩进']'换行,若非尾元素则添加逗号*/
        } else if (value && typeof value == 'object') {
            /*处理对象*/
            draw.push(
                tab + (formObj ? '"' + name + '":' : '') + '{' + line
            ); /*缩进'{' 然后换行*/
            var len = 0,
                i = 0;
            for (var key in value)
                len++;
            for (var key in value)
                notify(key, value[key], ++i == len, indent, true);
            draw.push(
                tab + '}' + (isLast ? line : ',' + line)
            ); /*缩进'}'换行,若非尾元素则添加逗号*/
        } else {
            if (typeof value == 'string') value = '"' + value + '"';
            draw.push(
                tab +
                (formObj ? '"' + name + '":' : '') +
                value +
                (isLast ? '' : ',') +
                line
            );
        }
    };
    var isLast = true,
        indent = 0;
    notify('', data, isLast, indent, false);
    return draw.join('');
};


console.log(process.argv)


// excel 的填写数据格式
// id	explain	text
// id	#说明	文字内容
// number	string	string

// node export_plot.js 导出地址 配置的表格地址 导出的文件名称
if (process.argv.length >= 3) {
    t_export_dis = process.argv[2];
    console.log("导出数据的位置 = ", t_export_dis)
    if (process.argv.length >= 4) {
        t_target_dis = process.argv[3];
    }

    if (process.argv.length >= 4) {
        t_target_dis = process.argv[3];
    }

    if (process.argv.length >= 5) {
        ts_target_dir = process.argv[5];
    }
    if (process.argv.length >= 6) {
        server_data_config = process.argv[6];
    }
    // 导出数据
    exportdata(t_export_dis, t_export_name, t_target_dis, ts_dir);
} else {
    console.log("请输入输出数据的地址");
}



// 压缩的数据 导出在目标目录下面
function compress_data(json_data, target_dir, file_name){ 
    return new Promise((resolve, reject) => {
        var binaryString = pako.deflate(JSON.stringify(json_data), { to: 'string' });
        setTimeout(() => {
            jsonfile.writeFile(`${target_dir}/${file_name}`, binaryString, function(err) {
                if (err) console.error(err)
                console.log(`导出文件成功`);
            });
            resolve();
        }, 3000);
    })
}


module.exports = exportdata;

==> hs_export_data-1.0.9.tgz: package/export_plot.js <==
const xlsx = require("node-xlsx");
const path = require('path')
const fs = require('fs')
const base_path = __dirname;
var parseString = require('xml2js').parseString;
var pako = require('pako');

const myReplace = function(data, f, e) { //吧f替换成e
    var reg = new RegExp(f, "g"); //创建正则RegExp对象   
    return data.replace(reg, e);
}

const jsonfile = require('jsonfile')
let t_export_dis = `${base_path}/./../`;
let t_export_name = "config.json";
let t_target_dis = `${base_path}/excels`;
let ts_target_dir = `${base_path}/ts`;
let ts_import = "";

let single_json_data_path = "./";


let server_data_config = [];

const server_config = {};

function exportdata(texport_dis, texport_name, ttarget_dis, ts_dir, server_data_config, single_json_data_path) {
        server_data_config = server_data_config;
        console.log("服务器端的配置数据", server_data_config);
        let export_dis = texport_dis;
        let export_name = texport_name;
        let target_dis = ttarget_dis;
        fs.readdir(target_dis, (error, files) => {
            console.log(target_dis);
            const total_data = {};
            var interfaces = "";
            var all_config = "const ConfigsName = {\n";

            var export_interface = " \n\n export { ";
            files.forEach((filename, index) => {
                let pathname = path.join(target_dis, filename);
                if (filename.indexOf(".DS_Store") == 0) {
                    return;
                }
                fs.stat(pathname, (err, stats) => { // 读取文件信息
                    if (err) {
                        return
                    }
    
                    if (stats.isFile()) {
    
                        console.log("pathname  ", pathname);
                        const list = xlsx.parse(fs.readFileSync(pathname));
                        const data = praseExcel(list, single_json_data_path);

                        const tname = filename.split(".")[0];
                        ts_import = ts_import + `import {${tname}} from "./ConfigInterface" \n`;
                        
                        const ts = prase_ts(list, tname);
                        interfaces = interfaces + ts;
                        if (data) {
                            total_data[`${tname}`] = data;

                            let is_server_config = false;
                            for(const file_name of server_data_config){
                                if(file_name == tname){
                                    is_server_config = true;
                                    break;
                                }
                            }
    
                            if(is_server_config){
                                server_config[`${tname}`] = data;
                            }
                        }

                        
    
                        export_interface = export_interface + tname + ","
                        all_config = all_config + `     ${tname}:"${tname}", \n`;

                        if (index == files.length - 1) {
                            setTimeout(() => {
                                 let data_str = JSON.stringify(total_data);
                                 writeFile(path.join(export_dis, export_name), JSON.parse(data_str), null, "\t");   
                                 writeFile(path.join(export_dis, "ServerConfig.json"), JSON.parse(JSON.stringify(server_config)), null, "\t");   

                                 const tpath = path.join(ts_dir, "ConfigInterface.ts");
                                 console.log("tpath ",tpath)
                                 fs.writeFile(tpath, `${interfaces} \n ${export_interface} };`, 'utf-8', (err) => {
                                     if (!err)      {         
                                         console.log(`导出文件${tpath}成功`);     
                                     }   
                                 }); 

                                 
                                 const t1path = path.join(ts_dir, "ConfigsName.ts");
                                 fs.writeFile(t1path, `${ts_import} \n\n ${all_config} }; \n export default ConfigsName;`, 'utf-8', (err) => {
                                     if (!err)      {         
                                         console.log(`导出文件${tpath}成功`);     
                                     }   
                                 }); 



                            }, 100);
                        }
                    }
    
    
                });
            });
        });


}

function parse_ts(list) {}

function praseExcel(list, single_json_data_path) {   
    var sheetArray  = [];      
    for (var i = 0; i < 1; i++) {        
        var excleData = list[i].data;        
        var typeArray =  excleData[2];        
        var keyArray =  excleData[0];      
        for (var j = 3; j < excleData.length; j++) {            
            var curData = excleData[j];            
            if (curData.length == 0) continue;            
            var item = changeObj(curData, typeArray, keyArray);     
            // console.log(item);        
            sheetArray.push(item);       
        }       
        if (sheetArray && single_json_data_path) {
            // console.log("sheetArray", sheetArray); 
            const file_name =  list[i].name + ".json"
            fs.writeFile(path.join(`${single_json_data_path}`,file_name), formatJson(JSON.stringify(sheetArray)), 'utf-8', (err) => {
                if (!err)      {         
                    console.log(`导出文件${file_name}成功`);     
                }   
            }); 
            // writeFile(path.join(`${single_json_data_path}`, list[i].name + ".json"), formatJson(JSON.stringify(sheetArray)), null, "\t");   
        }  
    }  
    return sheetArray;
}

function changeObj(curData, typeArray, keyArray) {    
    const plot_single = {};
    const plot_arraay = [
        []
    ];
    for (var i = 0; i < curData.length; i++)    { 
        const key = keyArray[i];
        if (key && key.substr(0, 1) !== "#") {
            plot_single[key] = changeValue(curData[i], typeArray[i]); 
        }
    }   


    return plot_single;
}

function changeValue(value, type) {   
    if (value == null || value == "null") return ""   
    if (type == "bool" || type == "b") return value;   
    if (type == "int" || type == "i") return Math.floor(value);   
    if (type == "bool") return value;
    if (type == "Number" || type == "number" || type == "n" || type == "float") return new Number(value);   
    if (type == "String" || type == "s" || type == "S" || type == "string") return value;
    if (type == "json" || type == "object" || type == "Array<string>" || type == "Array<number>" || type == "Array<any>" || type == "array" || type == "A" || type == "a") return JSON.parse(value); 
}

function prase_ts(list, file_name) {   
      
    let ts = "";

    for (var i = 0; i < 1; i++) {        
        var excleData = list[i].data;        
        var typeArray =  excleData[2];        
        var keyArray =  excleData[0];      
        ts = ts + single_ts_config_interface(file_name, typeArray, keyArray);
    }  
    
   
    return ts;
}

function single_ts_config_interface(name, typeArray, keyArray){
     if(!typeof typeArray != "array"){
        return;
     }
     var ts_interface = `interface ${name} { \n`;
     let key_index = 0;

     for(const type of typeArray){
        const key = keyArray[key_index];

        ts_interface = ts_interface + `    ${key} : ${type}; \n`;

        key_index = key_index + 1;
     }

     ts_interface =  `${ts_interface}} \n\n`;

     return ts_interface;
}

function writeFile(fileName, data) {  
    // fs.writeFile(fileName, data, 'utf-8', (err) => {
    //     if (!err)      {         
    //         console.log(`导出文件${fileName}成功`);     
    //     }   
    // }); 

    jsonfile.writeFile(fileName, data, function(err) {
        if (err) console.error(err)
        console.log(`导出文件${fileName}成功`);
    })

    compress_data(data, t_export_dis , "config.manifest");
}

var formatJson = function(txt, compress) {
    var indentChar = '    ';
    if (/^\s*$/.test(txt)) {
        console.log('数据为空,无法格式化! ');
        return;
    }
    try {
        var data = eval('(' + txt + ')');
    } catch (e) {
        console.log('数据源语法错误,格式化失败! 错误信息: ' + e.description, 'err');
        return;
    }
    var draw = [],
        last = false,
        This = this,
        line = compress ? '' : '\n',
        nodeCount = 0,
        maxDepth = 0;

    var notify = function(name, value, isLast, indent, formObj) {
        nodeCount++; /*节点计数*/
        for (var i = 0, tab = ''; i < indent; i++)
            tab += indentChar; /* 缩进HTML */
        tab = compress ? '' : tab; /*压缩模式忽略缩进*/
        maxDepth = ++indent; /*缩进递增并记录*/
        if (value && value.constructor == Array) {
            /*处理数组*/
            draw.push(
                tab + (formObj ? '"' + name + '":' : '') + '[' + line
            ); /*缩进'[' 然后换行*/
            for (var i = 0; i < value.length; i++)
                notify(i, value[i], i == value.length - 1, indent, false);
            draw.push(
                tab + ']' + (isLast ? line : ',' + line)
            ); /*缩进']'换行,若非尾元素则添加逗号*/
        } else if (value && typeof value == 'object') {
            /*处理对象*/
            draw.push(
                tab + (formObj ? '"' + name + '":' : '') + '{' + line
            ); /*缩进'{' 然后换行*/
            var len = 0,
                i = 0;
            for (var key in value)
                len++;
            for (var key in value)
                notify(key, value[key], ++i == len, indent, true);
            draw.push(
                tab + '}' + (isLast ? line : ',' + line)
            ); /*缩进'}'换行,若非尾元素则添加逗号*/
        } else {
            if (typeof value == 'string') value = '"' + value + '"';
            draw.push(
                tab +
                (formObj ? '"' + name + '":' : '') +
                value +
                (isLast ? '' : ',') +
                line
            );
        }
    };
    var isLast = true,
        indent = 0;
    notify('', data, isLast, indent, false);
    return draw.join('');
};


console.log(process.argv)


// excel 的填写数据格式
// id	explain	text
// id	#说明	文字内容
// number	string	string

// node export_plot.js 导出地址 配置的表格地址 导出的文件名称
if (process.argv.length >= 3) {
    t_export_dis = process.argv[2];
    console.log("导出数据的位置 = ", t_export_dis)
    if (process.argv.length >= 4) {
        t_target_dis = process.argv[3];
    }

    if (process.argv.length >= 4) {
        t_target_dis = process.argv[3];
    }

    if (process.argv.length >= 5) {
        ts_target_dir = process.argv[5];
    }
    if (process.argv.length >= 6) {
        server_data_config = process.argv[6];
    }

    if (process.argv.length >= 7) {
        single_json_data_path = process.argv[7];
    }
    

    // 导出数据
    // exportdata(t_export_dis, t_export_name, t_target_dis, ts_dir, single_json_data_path);
} else {
    console.log("请输入输出数据的地址");
}



// 压缩的数据 导出在目标目录下面
function compress_data(json_data, target_dir, file_name){ 
    return new Promise((resolve, reject) => {
        var binaryString = pako.deflate(JSON.stringify(json_data), { to: 'string' });
        setTimeout(() => {
            jsonfile.writeFile(`${target_dir}/${file_name}`, binaryString, function(err) {
                if (err) console.error(err)
                console.log(`导出文件成功`);
            });
            resolve();
        }, 3000);
    })
}


module.exports = exportdata;
